package java.security;

import javax.crypto.SecretKey;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.PasswordCallback;
import java.io.InputStream;
import java.security.cert.CertificateException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Date;
import java.security.cert.Certificate;

public abstract class KeyStoreSpi
{
    public KeyStoreSpi() {
    }
    
    public abstract Key engineGetKey(String p0, char[] p1) throws NoSuchAlgorithmException, UnrecoverableKeyException;
    
    public abstract Certificate[] engineGetCertificateChain(String p0);
    
    public abstract Certificate engineGetCertificate(String p0);
    
    public abstract Date engineGetCreationDate(String p0);
    
    public abstract void engineSetKeyEntry(String p0, Key p1, char[] p2, Certificate[] p3) throws KeyStoreException;
    
    public abstract void engineSetKeyEntry(String p0, byte[] p1, Certificate[] p2) throws KeyStoreException;
    
    public abstract void engineSetCertificateEntry(String p0, Certificate p1) throws KeyStoreException;
    
    public abstract void engineDeleteEntry(String p0) throws KeyStoreException;
    
    public abstract Enumeration<String> engineAliases();
    
    public abstract boolean engineContainsAlias(String p0);
    
    public abstract int engineSize();
    
    public abstract boolean engineIsKeyEntry(String p0);
    
    public abstract boolean engineIsCertificateEntry(String p0);
    
    public abstract String engineGetCertificateAlias(Certificate p0);
    
    public abstract void engineStore(OutputStream p0, char[] p1) throws IOException, NoSuchAlgorithmException, CertificateException;
    
    public void engineStore(KeyStore.LoadStoreParameter loadStoreParameter) throws IOException, NoSuchAlgorithmException, CertificateException {
        throw new UnsupportedOperationException();
    }
    
    public abstract void engineLoad(InputStream p0, char[] p1) throws IOException, NoSuchAlgorithmException, CertificateException;
    
    public void engineLoad(KeyStore.LoadStoreParameter loadStoreParameter) throws IOException, NoSuchAlgorithmException, CertificateException {
        if (loadStoreParameter == null) {
            this.engineLoad((InputStream)null, (char[])null);
            return;
        }
        if ((loadStoreParameter instanceof KeyStore.SimpleLoadStoreParameter)) {
            KeyStore.ProtectionParameter protectionParameter = loadStoreParameter.getProtectionParameter();
            char[] array;
            if ((protectionParameter instanceof KeyStore.PasswordProtection)) {
                array = ((KeyStore.PasswordProtection)protectionParameter).getPassword();
            }
            else if ((protectionParameter instanceof KeyStore.CallbackHandlerProtection)) {
                CallbackHandler callbackHandler = ((KeyStore.CallbackHandlerProtection)protectionParameter).getCallbackHandler();
                PasswordCallback passwordCallback = new PasswordCallback("Password: ", false);
                try {
                    callbackHandler.handle(new Callback[] { passwordCallback });
                }
                catch (UnsupportedCallbackException ex) {
                    throw new NoSuchAlgorithmException("Could not obtain password", ex);
                }
                array = passwordCallback.getPassword();
                passwordCallback.clearPassword();
                if (array == null) {
                    throw new NoSuchAlgorithmException("No password provided");
                }
            }
            else {
                throw new NoSuchAlgorithmException("ProtectionParameter must be PasswordProtection or CallbackHandlerProtection");
            }
            this.engineLoad(null, array);
            return;
        }
        else {
            throw new UnsupportedOperationException();
        }
    }
    
    public KeyStore.Entry engineGetEntry(String s, KeyStore.ProtectionParameter protectionParameter) throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableEntryException {
        if (!this.engineContainsAlias(s)) {
            return null;
        }
        if (protectionParameter == null) {
            if (this.engineIsCertificateEntry(s)) {
                return new KeyStore.TrustedCertificateEntry(this.engineGetCertificate(s));
            }
            throw new UnrecoverableKeyException("requested entry requires a password");
        }
        else {
            if ((protectionParameter instanceof KeyStore.PasswordProtection)) {
                if (this.engineIsCertificateEntry(s)) {
                    throw new UnsupportedOperationException("trusted certificate entries are not password-protected");
                }
                if (this.engineIsKeyEntry(s)) {
                    Key engineGetKey = this.engineGetKey(s, ((KeyStore.PasswordProtection)protectionParameter).getPassword());
                    if ((engineGetKey instanceof PrivateKey)) {
                        return new KeyStore.PrivateKeyEntry((PrivateKey)engineGetKey, this.engineGetCertificateChain(s));
                    }
                    if ((engineGetKey instanceof SecretKey)) {
                        return new KeyStore.SecretKeyEntry((SecretKey)engineGetKey);
                    }
                }
            }
            throw new UnsupportedOperationException();
        }
    }
    
    public void engineSetEntry(String s, KeyStore.Entry entry, KeyStore.ProtectionParameter protectionParameter) throws KeyStoreException {
        if (protectionParameter != null && !(protectionParameter instanceof KeyStore.PasswordProtection)) {
            throw new KeyStoreException("unsupported protection parameter");
        }
        KeyStore.PasswordProtection passwordProtection = null;
        if (protectionParameter != null) {
            passwordProtection = (KeyStore.PasswordProtection)protectionParameter;
        }
        if ((entry instanceof KeyStore.TrustedCertificateEntry)) {
            if (protectionParameter != null && passwordProtection.getPassword() != null) {
                throw new KeyStoreException("trusted certificate entries are not password-protected");
            }
            this.engineSetCertificateEntry(s, ((KeyStore.TrustedCertificateEntry)entry).getTrustedCertificate());
            return;
        }
        else if ((entry instanceof KeyStore.PrivateKeyEntry)) {
            if (passwordProtection == null || passwordProtection.getPassword() == null) {
                throw new KeyStoreException("non-null password required to create PrivateKeyEntry");
            }
            this.engineSetKeyEntry(s, ((KeyStore.PrivateKeyEntry)entry).getPrivateKey(), passwordProtection.getPassword(), ((KeyStore.PrivateKeyEntry)entry).getCertificateChain());
            return;
        }
        else if ((entry instanceof KeyStore.SecretKeyEntry)) {
            if (passwordProtection == null || passwordProtection.getPassword() == null) {
                throw new KeyStoreException("non-null password required to create SecretKeyEntry");
            }
            this.engineSetKeyEntry(s, ((KeyStore.SecretKeyEntry)entry).getSecretKey(), passwordProtection.getPassword(), (Certificate[])null);
            return;
        }
        else {
            throw new KeyStoreException("unsupported entry type: " + entry.getClass().getName());
        }
    }
    
    public boolean engineEntryInstanceOf(String s, Class<? extends KeyStore.Entry> clazz) {
        if (clazz == KeyStore.TrustedCertificateEntry.class) {
            return this.engineIsCertificateEntry(s);
        }
        if (clazz == KeyStore.PrivateKeyEntry.class) {
            return this.engineIsKeyEntry(s) && this.engineGetCertificate(s) != null;
        }
        return clazz == KeyStore.SecretKeyEntry.class && this.engineIsKeyEntry(s) && this.engineGetCertificate(s) == null;
    }
}
